@@ -1,6 +1,10 @@
 #!/usr/bin/env python
 
-# Author: Trevor Perrin
+# Authors: 
+#   Trevor Perrin
+#   Kees Bos - Added tests for XML-RPC
+#   Dimitris Moraitis - Anon ciphersuites
+#
 # See the LICENSE file for legal information regarding use of this file.
 
 import sys
@@ -23,9 +27,13 @@ from tlslite import TLSConnection, Fault, HandshakeSettings, \
 
 from tlslite.errors import *
 from tlslite.utils.cryptomath import prngName
+import xmlrpclib
+from tlslite import *
 
 try:
-    from TACKpy import TACK, TACK_Break_Sig
+    from tack.structures.Tack import Tack
+    from tack.structures.TackBreakSig import TackBreakSig
+    
 except ImportError:
     pass
 
@@ -81,11 +89,18 @@ def clientTestCmd(argv):
 
     badFault = False
     
-    print "Test 1 - good X509"
+    print "Test 0 - anonymous handshake"
     connection = connect()
-    connection.handshakeClientCert()
+    connection.handshakeClientAnonymous()
+    testConnClient(connection)
+    connection.close()
+        
+    print "Test 1 - good X509 (plus SNI)"
+    connection = connect()
+    connection.handshakeClientCert(serverName=address[0])
     testConnClient(connection)
     assert(isinstance(connection.session.serverCertChain, X509CertChain))
+    assert(connection.session.serverName == address[0])
     connection.close()
 
     print "Test 1.a - good X509, SSLv3"
@@ -99,56 +114,49 @@ def clientTestCmd(argv):
     connection.close()    
     
     if tackpyLoaded:
-        print "Test 2.a - good X.509, good TACK"
+                    
+        settings = HandshakeSettings()
+        settings.useExperimentalTackExtension = True
+        
+        print "Test 2.a - good X.509, TACK and Break Sigs"
         connection = connect()
-        connection.handshakeClientCert(reqTack=True, 
-         checker=Checker(tackID="BHNHV.8Q3RM.Q74EA.4X8F1.FMQEQ", hardTack=True))
+        connection.handshakeClientCert(settings=settings)
+        assert(connection.session.tackExt.tack.getTackId() == "rrted.ptvtl.d2uiq.ox2xe.w4ss3")
+        assert(connection.session.tackExt.break_sigs[0].getTackId() == "nxt3o.zoovj.xq5qk.mue76.wmx6t")
+        assert(connection.session.tackExt.break_sigs[1].getTackId() == "neuda.jg26x.ixmom.wzdry.qxdii")
+        assert(connection.session.tackExt.pin_activation == False)
         testConnClient(connection)    
         connection.close()    
 
-        try:
-            print "Test 2.b - good X.509, \"wrong\" TACK"
-            connection = connect()
-            connection.handshakeClientCert(reqTack=True, 
-             checker=Checker(tackID="B4444.EQ61B.F34EL.9KKLN.3WEW5", hardTack=True))
-            assert(False)
-        except TLSTackMismatchError:
-            pass
-
-        print "Test 2.c - good X.509, \"wrong\" TACK but break signature (hardTack)"
+        print "Test 2.b - good X.509, TACK without Break Sigs"
         connection = connect()
-        try:
-            connection.handshakeClientCert(reqTack=True, 
-                checker=Checker(tackID="BHNHV.8Q3RM.Q74EA.4X8F1.FMQEQ", hardTack=True))
-            assert(False)
-        except TLSTackBreakError:
-            pass
-
-        print "Test 2.d - good X.509, \"wrong\" TACK but break signature (not hardTack)"
-        connection = connect()
-        connection.handshakeClientCert(reqTack=True, 
-            checker=Checker(tackID="BHNHV.8Q3RM.Q74EA.4X8F1.FMQEQ", hardTack=False))
+        connection.handshakeClientCert(settings=settings)
+        assert(connection.session.tackExt.tack.getTackId() == "rrted.ptvtl.d2uiq.ox2xe.w4ss3")
+        assert(not connection.session.tackExt.break_sigs)
+        assert(connection.session.tackExt.pin_activation == True)        
         testConnClient(connection)    
         connection.close()    
 
-        print "Test 2.e - good X.509, TACK unrelated to cert chain"
+        print "Test 2.c - good X.509, Break Sigs without TACK"
+        connection = connect()
+        connection.handshakeClientCert(settings=settings)
+        assert(connection.session.tackExt.tack == None)
+        assert(connection.session.tackExt.break_sigs[0].getTackId() == "nxt3o.zoovj.xq5qk.mue76.wmx6t")
+        assert(connection.session.tackExt.break_sigs[1].getTackId() == "neuda.jg26x.ixmom.wzdry.qxdii")
+        assert(connection.session.tackExt.pin_activation == True)
+        testConnClient(connection)    
+        connection.close()    
+
+        print "Test 2.d - good X.509, TACK unrelated to cert chain"
         connection = connect()
         try:
-            connection.handshakeClientCert(reqTack=True)
+            connection.handshakeClientCert(settings=settings)
             assert(False)
-        except TLSLocalAlert as alert:
-            assert(alert.description == AlertDescription.handshake_failure)
+        except TLSLocalAlert, alert:
+            if alert.description != AlertDescription.illegal_parameter:
+                raise        
         connection.close()
 
-        try:
-            print "Test 2.f - good X.509, no TACK but expected"
-            connection = connect()
-            connection.handshakeClientCert(reqTack=True, 
-                checker=Checker(tackID="B4444.EQ61B.F34EL.9KKLN.3WEW5", hardTack=False))
-            assert(False)
-        except TLSTackMissingError:
-            pass
-
     print "Test 3 - good SRP"
     connection = connect()
     connection.handshakeClientSRP("test", "password")
@@ -231,24 +239,26 @@ def clientTestCmd(argv):
             print "  BAD FAULT %s: %s" % (Fault.faultNames[fault], str(e))
             badFault = True
 
-    print "Test 18 - good SRP, prepare to resume..."
+    print "Test 18 - good SRP, prepare to resume... (plus SNI)"
     connection = connect()
-    connection.handshakeClientSRP("test", "password")
+    connection.handshakeClientSRP("test", "password", serverName=address[0])
     testConnClient(connection)
     connection.close()
     session = connection.session
 
-    print "Test 19 - resumption"
+    print "Test 19 - resumption (plus SNI)"
     connection = connect()
-    connection.handshakeClientSRP("test", "garbage", session=session)
+    connection.handshakeClientSRP("test", "garbage", serverName=address[0], 
+                                    session=session)
     testConnClient(connection)
     #Don't close! -- see below
 
-    print "Test 20 - invalidated resumption"
+    print "Test 20 - invalidated resumption (plus SNI)"
     connection.sock.close() #Close the socket without a close_notify!
     connection = connect()
     try:
-        connection.handshakeClientSRP("test", "garbage", session=session)
+        connection.handshakeClientSRP("test", "garbage", 
+                        serverName=address[0], session=session)
         assert(False)
     except TLSRemoteAlert, alert:
         if alert.description != AlertDescription.bad_record_mac:
@@ -267,8 +277,9 @@ def clientTestCmd(argv):
             htmlBody = open(os.path.join(dir, "index.html")).read()
             fingerprint = None
             for y in range(2):
+                checker =Checker(x509Fingerprint=fingerprint)
                 h = HTTPTLSConnection(\
-                        address[0], address[1], x509Fingerprint=fingerprint)
+                        address[0], address[1], checker=checker)
                 for x in range(3):
                     h.request("GET", "/index.html")
                     r = h.getresponse()
@@ -335,18 +346,35 @@ def clientTestCmd(argv):
 
             assert(h == "hello"*10000)
             connection.close()
+            
+    print 'Test 24 - good standard XMLRPC https client'
+    address = address[0], address[1]+1
+    server = xmlrpclib.Server('https://%s:%s' % address)
+    assert server.add(1,2) == 3
+    assert server.pow(2,4) == 16
 
-    print "Test 24 - Internet servers test"
+    print 'Test 25 - good tlslite XMLRPC client'
+    transport = XMLRPCTransport(ignoreAbruptClose=True)
+    server = xmlrpclib.Server('https://%s:%s' % address, transport)
+    assert server.add(1,2) == 3
+    assert server.pow(2,4) == 16
+
+    print 'Test 26 - good XMLRPC ignored protocol'
+    server = xmlrpclib.Server('http://%s:%s' % address, transport)
+    assert server.add(1,2) == 3
+    assert server.pow(2,4) == 16
+
+    print "Test 27 - Internet servers test"
     try:
         i = IMAP4_TLS("cyrus.andrew.cmu.edu")
         i.login("anonymous", "anonymous@anonymous.net")
         i.logout()
-        print "Test 24: IMAP4 good"
+        print "Test 28: IMAP4 good"
         p = POP3_TLS("pop.gmail.com")
         p.quit()
-        print "Test 24: POP3 good"
+        print "Test 29: POP3 good"
     except socket.error, e:
-        print "Non-critical error: socket error trying to reach internet server: ", e
+        print "Non-critical error: socket error trying to reach internet server: ", e   
 
     if not badFault:
         print "Test succeeded"
@@ -383,6 +411,12 @@ def serverTestCmd(argv):
     def connect():
         return TLSConnection(lsock.accept()[0])
 
+    print "Test 0 - Anonymous server handshake"
+    connection = connect()
+    connection.handshakeServer(anon=True)
+    testConnServer(connection)    
+    connection.close() 
+    
     print "Test 1 - good X.509"
     x509Cert = X509().parse(open(os.path.join(dir, "serverX509Cert.pem")).read())
     x509Chain = X509CertChain([x509Cert])
@@ -391,6 +425,7 @@ def serverTestCmd(argv):
 
     connection = connect()
     connection.handshakeServer(certChain=x509Chain, privateKey=x509Key)
+    assert(connection.session.serverName == address[0])    
     testConnServer(connection)    
     connection.close()
 
@@ -404,56 +439,45 @@ def serverTestCmd(argv):
     connection.close()        
     
     if tackpyLoaded:
-        # TACK1 and TACK2 are both "good" TACKs, one targetting, the key,
-        # one the hash
-        tack1 = TACK()
-        tack1.parsePem(open("./TACK1.pem", "rU").read())
-        tack2 = TACK()
-        tack2.parsePem(open("./TACK2.pem", "rU").read())
-        tackUnrelated = TACK()
-        tackUnrelated.parsePem(open("./TACKunrelated.pem", "rU").read())    
-        breakSigs = TACK_Break_Sig.parsePemList(
+        tack = Tack.createFromPem(open("./TACK1.pem", "rU").read())
+        tackUnrelated = Tack.createFromPem(open("./TACKunrelated.pem", "rU").read())    
+        breakSigs = TackBreakSig.createFromPemList(
             open("./TACK_Break_Sigs.pem").read())
-        breakSigsActual = TACK_Break_Sig.parsePemList(
-            open("./TACK_Break_Sigs_TACK1.pem").read())    
+            
+        settings = HandshakeSettings()
+        settings.useExperimentalTackExtension = True
 
-        print "Test 2.a - good X.509, good TACK"
+        print "Test 2.a - good X.509, TACK and Break Sigs"
         connection = connect()
         connection.handshakeServer(certChain=x509Chain, privateKey=x509Key,
-            tack=tack1, breakSigs=breakSigs)
+            tack=tack, breakSigs=breakSigs, settings=settings)
         testConnServer(connection)    
         connection.close()        
 
-        print "Test 2.b - good X.509, \"wrong\" TACK"
+        print "Test 2.b - good X.509, TACK without Break Sigs"
         connection = connect()
         connection.handshakeServer(certChain=x509Chain, privateKey=x509Key,
-            tack=tack1)
+            tack=tack, pinActivation=True, settings=settings)
+        testConnServer(connection)    
         connection.close()        
 
-        print "Test 2.c - good X.509, \"wrong\" TACK but break signature (hardTack)"
+        print "Test 2.c - good X.509, Break Sigs without TACK"
         connection = connect()
         connection.handshakeServer(certChain=x509Chain, privateKey=x509Key,
-            tack=tack2, breakSigs=breakSigsActual)
-
-        print "Test 2.d - good X.509, \"wrong\" TACK but break signature (not hardTack)"
-        connection = connect()
-        connection.handshakeServer(certChain=x509Chain, privateKey=x509Key,
-            tack=tack2, breakSigs=breakSigsActual)
+            breakSigs=breakSigs, pinActivation=True, settings=settings)
         testConnServer(connection)    
-        connection.close()
+        connection.close()        
 
-        print "Test 2.e - good X.509, TACK unrelated to cert chain"
+        print "Test 2.d - good X.509, TACK unrelated to cert chain"
         connection = connect()
         try:
             connection.handshakeServer(certChain=x509Chain, privateKey=x509Key,
-                tack=tackUnrelated)
-        except TLSRemoteAlert as alert:
-            assert(alert.description == AlertDescription.handshake_failure)
-
-        print "Test 2.f - good X.509, no TACK but expected"
-        connection = connect()
-        connection.handshakeServer(certChain=x509Chain, privateKey=x509Key)
-        connection.close()        
+                tack=tackUnrelated, breakSigs=breakSigs, settings=settings)
+            assert(False)
+        except TLSRemoteAlert, alert:
+            if alert.description != AlertDescription.illegal_parameter:
+                raise        
+    
 
     print "Test 3 - good SRP"
     verifierDB = VerifierDB()
@@ -539,12 +563,14 @@ def serverTestCmd(argv):
     sessionCache = SessionCache()
     connection = connect()
     connection.handshakeServer(verifierDB=verifierDB, sessionCache=sessionCache)
+    assert(connection.session.serverName == address[0])    
     testConnServer(connection)
     connection.close()
 
     print "Test 19 - resumption"
     connection = connect()
     connection.handshakeServer(verifierDB=verifierDB, sessionCache=sessionCache)
+    assert(connection.session.serverName == address[0])
     testConnServer(connection)    
     #Don't close! -- see next test
 
@@ -636,6 +662,33 @@ def serverTestCmd(argv):
             connection.write(h)
             connection.close()
 
+    print "Tests 24-26 - XMLRPXC server"
+    address = address[0], address[1]+1
+    class Server(TLSXMLRPCServer):
+
+        def handshake(self, tlsConnection):
+          try:
+              tlsConnection.handshakeServer(certChain=x509Chain,
+                                            privateKey=x509Key,
+                                            sessionCache=sessionCache)
+              tlsConnection.ignoreAbruptClose = True
+              return True
+          except TLSError, error:
+              print "Handshake failure:", str(error)
+              return False
+
+    class MyFuncs:
+        def pow(self, x, y): return pow(x, y)
+        def add(self, x, y): return x + y
+
+    server = Server(address)
+    server.register_instance(MyFuncs())
+    #sa = server.socket.getsockname()
+    #print "Serving HTTPS on", sa[0], "port", sa[1]
+    for i in range(6):
+        server.handle_request()
+
+
     print "Test succeeded"
 
 
@@ -648,4 +701,3 @@ if __name__ == '__main__':
         serverTestCmd(sys.argv[2:])
     else:
         printUsage("Unknown command: %s" % sys.argv[1])
-      
\ No newline at end of file
