@@ -1,6 +1,8 @@
 # Authors: 
 #   Trevor Perrin
 #   Google - defining ClientCertificateType
+#   Google (adapted by Sam Rushing) - NPN support
+#   Dimitris Moraitis - Anon ciphersuites
 #
 # See the LICENSE file for legal information regarding use of this file.
 
@@ -27,6 +29,7 @@ class HandshakeType:
     certificate_verify = 15
     client_key_exchange = 16
     finished = 20
+    next_protocol = 67
 
 class ContentType:
     change_cipher_spec = 20
@@ -35,11 +38,16 @@ class ContentType:
     application_data = 23
     all = (20,21,22,23)
 
-class ExtensionType:    # RFC 4366
+class ExtensionType:    # RFC 6066 / 4366
+    server_name = 0     # RFC 6066 / 4366
     srp = 12            # RFC 5054  
     cert_type = 9       # RFC 6091
     tack = 0xF300
     break_sigs = 0xF301
+    supports_npn = 13172
+    
+class NameType:
+    host_name = 0
 
 class AlertLevel:
     warning = 1
@@ -121,6 +129,9 @@ class CipherSuite:
     TLS_RSA_WITH_AES_256_CBC_SHA = 0x0035
     TLS_RSA_WITH_RC4_128_SHA = 0x0005
 
+    TLS_DH_ANON_WITH_AES_128_CBC_SHA = 0x0034
+    TLS_DH_ANON_WITH_AES_256_CBC_SHA = 0x003A
+    
     srpSuites = []
     srpSuites.append(TLS_SRP_SHA_WITH_3DES_EDE_CBC_SHA)
     srpSuites.append(TLS_SRP_SHA_WITH_AES_128_CBC_SHA)
@@ -182,6 +193,20 @@ class CipherSuite:
                 suites.append(CipherSuite.TLS_RSA_WITH_3DES_EDE_CBC_SHA)
         return suites
 
+    anonSuites = []
+    anonSuites.append(TLS_DH_ANON_WITH_AES_128_CBC_SHA)
+    anonSuites.append(TLS_DH_ANON_WITH_AES_256_CBC_SHA)
+    
+    @staticmethod
+    def getAnonSuites(ciphers):
+        suites = []
+        for cipher in ciphers:
+            if cipher == "aes128":
+                suites.append(CipherSuite.TLS_DH_ANON_WITH_AES_128_CBC_SHA)
+            elif cipher == "aes256":
+                suites.append(CipherSuite.TLS_DH_ANON_WITH_AES_256_CBC_SHA)
+            return suites   
+    
     tripleDESSuites = []
     tripleDESSuites.append(TLS_SRP_SHA_WITH_3DES_EDE_CBC_SHA)
     tripleDESSuites.append(TLS_SRP_SHA_RSA_WITH_3DES_EDE_CBC_SHA)
@@ -191,11 +216,13 @@ class CipherSuite:
     aes128Suites.append(TLS_SRP_SHA_WITH_AES_128_CBC_SHA)
     aes128Suites.append(TLS_SRP_SHA_RSA_WITH_AES_128_CBC_SHA)
     aes128Suites.append(TLS_RSA_WITH_AES_128_CBC_SHA)
+    aes128Suites.append(TLS_DH_ANON_WITH_AES_128_CBC_SHA)
 
     aes256Suites = []
     aes256Suites.append(TLS_SRP_SHA_WITH_AES_256_CBC_SHA)
     aes256Suites.append(TLS_SRP_SHA_RSA_WITH_AES_256_CBC_SHA)
     aes256Suites.append(TLS_RSA_WITH_AES_256_CBC_SHA)
+    aes256Suites.append(TLS_DH_ANON_WITH_AES_256_CBC_SHA)
 
     rc4Suites = []
     rc4Suites.append(TLS_RSA_WITH_RC4_128_SHA)
