@@ -1,6 +1,8 @@
 # Authors: 
 #   Trevor Perrin
 #   Google - added reqCAs parameter
+#   Google (adapted by Sam Rushing) - NPN support
+#   Dimitris Moraitis - Anon ciphersuites
 #
 # See the LICENSE file for legal information regarding use of this file.
 
@@ -60,9 +62,75 @@ class TLSConnection(TLSRecordLayer):
     # Client Handshake Functions
     #*********************************************************
 
+    def handshakeClientAnonymous(self, session=None, settings=None, 
+                                checker=None, serverName="",
+                                async=False):
+        """Perform an anonymous handshake in the role of client.
+
+        This function performs an SSL or TLS handshake using an
+        anonymous Diffie Hellman ciphersuite.
+        
+        Like any handshake function, this can be called on a closed
+        TLS connection, or on a TLS connection that is already open.
+        If called on an open connection it performs a re-handshake.
+
+        If the function completes without raising an exception, the
+        TLS connection will be open and available for data transfer.
+
+        If an exception is raised, the connection will have been
+        automatically closed (if it was ever open).
+
+        @type session: L{tlslite.Session.Session}
+        @param session: A TLS session to attempt to resume.  If the
+        resumption does not succeed, a full handshake will be
+        performed.
+
+        @type settings: L{tlslite.HandshakeSettings.HandshakeSettings}
+        @param settings: Various settings which can be used to control
+        the ciphersuites, certificate types, and SSL/TLS versions
+        offered by the client.
+
+        @type checker: L{tlslite.Checker.Checker}
+        @param checker: A Checker instance.  This instance will be
+        invoked to examine the other party's authentication
+        credentials, if the handshake completes succesfully.
+        
+        @type serverName: string
+        @param serverName: The ServerNameIndication TLS Extension.
+
+        @type async: bool
+        @param async: If False, this function will block until the
+        handshake is completed.  If True, this function will return a
+        generator.  Successive invocations of the generator will
+        return 0 if it is waiting to read from the socket, 1 if it is
+        waiting to write to the socket, or will raise StopIteration if
+        the handshake operation is completed.
+
+        @rtype: None or an iterable
+        @return: If 'async' is True, a generator object will be
+        returned.
+
+        @raise socket.error: If a socket error occurs.
+        @raise tlslite.errors.TLSAbruptCloseError: If the socket is closed
+        without a preceding alert.
+        @raise tlslite.errors.TLSAlert: If a TLS alert is signalled.
+        @raise tlslite.errors.TLSAuthenticationError: If the checker
+        doesn't like the other party's authentication credentials.
+        """
+        handshaker = self._handshakeClientAsync(anonParams=(True),
+                                                session=session,
+                                                settings=settings,
+                                                checker=checker,
+                                                serverName=serverName)
+        if async:
+            return handshaker
+        for result in handshaker:
+            pass
+
     def handshakeClientSRP(self, username, password, session=None,
                            settings=None, checker=None, 
-                           reqTack=False, async=False):
+                           reqTack=True, serverName="",
+                           async=False):
         """Perform an SRP handshake in the role of client.
 
         This function performs a TLS/SRP handshake.  SRP mutually
@@ -102,7 +170,10 @@ class TLSConnection(TLSRecordLayer):
 
         @type reqTack: bool
         @param reqTack: Whether or not to send a "tack" TLS Extension, 
-        requesting the server return a TACK_Extension if it has one.
+        requesting the server return a TackExtension if it has one.
+
+        @type serverName: string
+        @param serverName: The ServerNameIndication TLS Extension.
 
         @type async: bool
         @param async: If False, this function will block until the
@@ -125,7 +196,7 @@ class TLSConnection(TLSRecordLayer):
         """
         handshaker = self._handshakeClientAsync(srpParams=(username, password),
                         session=session, settings=settings, checker=checker,
-                        reqTack=reqTack)
+                        reqTack=reqTack, serverName=serverName)
         # The handshaker is a Python Generator which executes the handshake.
         # It allows the handshake to be run in a "piecewise", asynchronous
         # fashion, returning 1 when it is waiting to able to write, 0 when
@@ -140,7 +211,7 @@ class TLSConnection(TLSRecordLayer):
 
     def handshakeClientCert(self, certChain=None, privateKey=None,
                             session=None, settings=None, checker=None,
-                            reqTack=False, async=False):
+                            reqTack=True, serverName="", async=False):
         """Perform a certificate-based handshake in the role of client.
 
         This function performs an SSL or TLS handshake.  The server
@@ -188,7 +259,10 @@ class TLSConnection(TLSRecordLayer):
         
         @type reqTack: bool
         @param reqTack: Whether or not to send a "tack" TLS Extension, 
-        requesting the server return a TACK_Extension if it has one.        
+        requesting the server return a TackExtension if it has one.        
+
+        @type serverName: string
+        @param serverName: The ServerNameIndication TLS Extension.
 
         @type async: bool
         @param async: If False, this function will block until the
@@ -211,7 +285,8 @@ class TLSConnection(TLSRecordLayer):
         """
         handshaker = self._handshakeClientAsync(certParams=(certChain,
                         privateKey), session=session, settings=settings,
-                        checker=checker, reqTack=reqTack)
+                        checker=checker, serverName=serverName, 
+                        reqTack=reqTack)
         # The handshaker is a Python Generator which executes the handshake.
         # It allows the handshake to be run in a "piecewise", asynchronous
         # fashion, returning 1 when it is waiting to able to write, 0 when
@@ -225,21 +300,23 @@ class TLSConnection(TLSRecordLayer):
             pass
 
 
-    def _handshakeClientAsync(self, srpParams=(), certParams=(),
+    def _handshakeClientAsync(self, srpParams=(), certParams=(), anonParams=(),
                              session=None, settings=None, checker=None,
-                             reqTack=False):
+                             serverName="", reqTack=True):
 
         handshaker = self._handshakeClientAsyncHelper(srpParams=srpParams,
                 certParams=certParams,
+                anonParams=anonParams,
                 session=session,
                 settings=settings,
+                serverName=serverName,
                 reqTack=reqTack)
         for result in self._handshakeWrapperAsync(handshaker, checker):
             yield result
 
 
-    def _handshakeClientAsyncHelper(self, srpParams, certParams, 
-                               session, settings, reqTack):
+    def _handshakeClientAsyncHelper(self, srpParams, certParams, anonParams,
+                               session, settings, serverName, reqTack):
         
         self._handshakeStart(client=True)
 
@@ -249,13 +326,18 @@ class TLSConnection(TLSRecordLayer):
         clientCertChain = None  # certParams[0]
         privateKey = None       # certParams[1]
 
-        # Allow only one of (srpParams, certParams)
+        # Allow only one of (srpParams, certParams, anonParams)
         if srpParams:
             assert(not certParams)
+            assert(not anonParams)
             srpUsername, password = srpParams
         if certParams:
             assert(not srpParams)
+            assert(not anonParams)            
             clientCertChain, privateKey = certParams
+        if anonParams:
+            assert(not srpParams)         
+            assert(not certParams)
 
         #Validate parameters
         if srpUsername and not password:
@@ -266,8 +348,11 @@ class TLSConnection(TLSRecordLayer):
             raise ValueError("Caller passed a certChain but no privateKey")
         if privateKey and not clientCertChain:
             raise ValueError("Caller passed a privateKey but no certChain")
-        if reqTack and not tackpyLoaded:
-            raise ValueError("TACKpy is not loaded")
+        if reqTack:
+            if not tackpyLoaded:
+                reqTack = False
+            if not settings or not settings.useExperimentalTackExtension:
+                reqTack = False
         
         # Validates the settings and filters out any unsupported ciphers
         # or crypto libraries that were requested        
@@ -287,9 +372,11 @@ class TLSConnection(TLSRecordLayer):
             # non-empty sessionID
             if not session.valid():
                 session = None #ignore non-resumable sessions...
-            elif session.resumable and \
-                    (session.srpUsername != srpUsername):
-                raise ValueError("Session username doesn't match")
+            elif session.resumable: 
+                if session.srpUsername != srpUsername:
+                    raise ValueError("Session username doesn't match")
+                if session.serverName != serverName:
+                    raise ValueError("Session servername doesn't match")
 
         #Add Faults to parameters
         if srpUsername and self.fault == Fault.badUsername:
@@ -308,7 +395,7 @@ class TLSConnection(TLSRecordLayer):
         # Send the ClientHello.
         for result in self._clientSendClientHello(settings, session, 
                                         srpUsername, srpParams, certParams,
-                                        reqTack):
+                                        anonParams, serverName, reqTack):
             if result in (0,1): yield result
             else: break
         clientHello = result
@@ -341,8 +428,16 @@ class TLSConnection(TLSRecordLayer):
                     serverHello.tackExt):                
                 if result in (0,1): yield result
                 else: break                
-            (premasterSecret, serverCertChain, tackExt) = result               
-                
+            (premasterSecret, serverCertChain, tackExt) = result
+        #If the server selected an anonymous ciphersuite, the client
+        #finishes reading the post-ServerHello messages.
+        elif cipherSuite in CipherSuite.anonSuites:
+            for result in self._clientAnonKeyExchange(settings, cipherSuite,
+                                    clientHello.random, serverHello.random):
+                if result in (0,1): yield result
+                else: break
+            (premasterSecret, serverCertChain, tackExt) = result     
+               
         #If the server selected a certificate-based RSA ciphersuite,
         #the client finishes reading the post-ServerHello messages. If 
         #a CertificateRequest message was sent, the client responds with
@@ -374,20 +469,23 @@ class TLSConnection(TLSRecordLayer):
         self.session = Session()
         self.session.create(masterSecret, serverHello.session_id, cipherSuite,
             srpUsername, clientCertChain, serverCertChain,
-            tackExt)
+            tackExt, serverHello.tackExt!=None, clientHello.server_name)
         self._handshakeDone(resumed=False)
 
 
     def _clientSendClientHello(self, settings, session, srpUsername,
-                                srpParams, certParams, reqTack):
+                                srpParams, certParams, anonParams, 
+                                serverName, reqTack):
         #Initialize acceptable ciphersuites
         cipherSuites = [CipherSuite.TLS_EMPTY_RENEGOTIATION_INFO_SCSV]
         if srpParams:
             cipherSuites += CipherSuite.getSrpAllSuites(settings.cipherNames)
         elif certParams:
             cipherSuites += CipherSuite.getCertSuites(settings.cipherNames)
+        elif anonParams:
+            cipherSuites += CipherSuite.getAnonSuites(settings.cipherNames)
         else:
-            cipherSuites += CipherSuite.getCertSuites(settings.cipherNames)
+            assert(False)
 
         #Initialize acceptable certificate types
         certificateTypes = settings._getCertificateTypes()
@@ -404,7 +502,7 @@ class TLSConnection(TLSRecordLayer):
                 clientHello.create(settings.maxVersion, getRandomBytes(32),
                                    session.sessionID, cipherSuites,
                                    certificateTypes, session.srpUsername,
-                                   reqTack)
+                                   reqTack, False, session.serverName)
 
         #Or send ClientHello (without)
         else:
@@ -412,7 +510,7 @@ class TLSConnection(TLSRecordLayer):
             clientHello.create(settings.maxVersion, getRandomBytes(32),
                                createByteArraySequence([]), cipherSuites,
                                certificateTypes, srpUsername,
-                               reqTack)
+                               reqTack, False, serverName)
         for result in self._sendMsg(clientHello):
             yield result
         yield clientHello
@@ -458,11 +556,17 @@ class TLSConnection(TLSRecordLayer):
                 AlertDescription.illegal_parameter,
                 "Server responded with incorrect compression method"):
                 yield result
-        if serverHello.tackExt and not clientHello.tack:
-            for result in self._sendError(\
-                AlertDescription.illegal_parameter,
-                "Server responded with unrequested TACK Extension"):
-                yield result
+        if serverHello.tackExt:            
+            if not clientHello.tack:
+                for result in self._sendError(\
+                    AlertDescription.illegal_parameter,
+                    "Server responded with unrequested Tack Extension"):
+                    yield result
+            if not serverHello.tackExt.verifySignatures():
+                for result in self._sendError(\
+                    AlertDescription.decrypt_error,
+                    "TackExtension contains an invalid signature"):
+                    yield result
         yield serverHello
  
     def _clientResume(self, session, serverHello, clientRandom, 
@@ -719,9 +823,40 @@ class TLSConnection(TLSRecordLayer):
                 yield result
         yield (premasterSecret, serverCertChain, clientCertChain, tackExt)
 
+    def _clientAnonKeyExchange(self, settings, cipherSuite, clientRandom, 
+                               serverRandom):
+        for result in self._getMsg(ContentType.handshake,
+                HandshakeType.server_key_exchange, cipherSuite):
+            if result in (0,1): yield result
+            else: break
+        serverKeyExchange = result
+
+        for result in self._getMsg(ContentType.handshake,
+                HandshakeType.server_hello_done):
+            if result in (0,1): yield result
+            else: break
+        serverHelloDone = result
+            
+        #calculate Yc
+        dh_p = serverKeyExchange.dh_p
+        dh_g = serverKeyExchange.dh_g
+        dh_Xc = bytesToNumber(getRandomBytes(32))
+        dh_Ys = serverKeyExchange.dh_Ys
+        dh_Yc = powMod(dh_g, dh_Xc, dh_p)
+        
+        #Send ClientKeyExchange
+        for result in self._sendMsg(\
+                ClientKeyExchange(cipherSuite, self.version).createDH(dh_Yc)):
+            yield result
+            
+        #Calculate premaster secret
+        S = powMod(dh_Ys, dh_Xc, dh_p)
+        premasterSecret = numberToBytes(S)
+                     
+        yield (premasterSecret, None, None)
+        
     def _clientFinished(self, premasterSecret, clientRandom, serverRandom,
-                        cipherSuite, cipherImplementations):
-                        
+                        cipherSuite, cipherImplementations):                   
         masterSecret = calcMasterSecret(self.version, premasterSecret,
                             clientRandom, serverRandom)
         self._calcPendingStates(cipherSuite, masterSecret, 
@@ -762,10 +897,11 @@ class TLSConnection(TLSRecordLayer):
          
             # If there's a TACK (whether via TLS or TACK Cert), check that it
             # matches the cert chain   
-            if tackExt and tackExt.tack and \
-                                not certChain.checkTack(tackExt.tack):
-                for result in self._sendError(AlertDescription.handshake_failure,
-                        "Other party's TACK doesn't match their cert chain"):
+            if tackExt and (tackExt.tack and 
+                            not certChain.checkTack(tackExt.tack)):
+                for result in self._sendError(
+                        AlertDescription.illegal_parameter,
+                        "Other party's TACK doesn't match their public key"):
                         yield result
 
         yield publicKey, certChain, tackExt
@@ -779,7 +915,9 @@ class TLSConnection(TLSRecordLayer):
     def handshakeServer(self, verifierDB=None,
                         certChain=None, privateKey=None, reqCert=False,
                         sessionCache=None, settings=None, checker=None,
-                        reqCAs = None, tack=None, breakSigs=None):
+                        reqCAs = None, 
+                        tack=None, breakSigs=None, pinActivation=False,
+                        nextProtos=None, anon=False):
         """Perform a handshake in the role of server.
 
         This function performs an SSL or TLS handshake.  Depending on
@@ -852,14 +990,19 @@ class TLSConnection(TLSRecordLayer):
         """
         for result in self.handshakeServerAsync(verifierDB,
                 certChain, privateKey, reqCert, sessionCache, settings,
-                checker, reqCAs, tack=tack, breakSigs=breakSigs):
+                checker, reqCAs, 
+                tack=tack, breakSigs=breakSigs, pinActivation=pinActivation, 
+                nextProtos=nextProtos, anon=anon):
             pass
 
 
     def handshakeServerAsync(self, verifierDB=None,
                              certChain=None, privateKey=None, reqCert=False,
                              sessionCache=None, settings=None, checker=None,
-                             reqCAs=None, tack=None, breakSigs=None):
+                             reqCAs=None, 
+                             tack=None, breakSigs=None, pinActivation=False,
+                             nextProtos=None, anon=False
+                             ):
         """Start a server handshake operation on the TLS connection.
 
         This function returns a generator which behaves similarly to
@@ -875,18 +1018,22 @@ class TLSConnection(TLSRecordLayer):
             verifierDB=verifierDB, certChain=certChain,
             privateKey=privateKey, reqCert=reqCert,
             sessionCache=sessionCache, settings=settings, 
-            reqCAs=reqCAs, tack=tack, breakSigs=breakSigs)
+            reqCAs=reqCAs, 
+            tack=tack, breakSigs=breakSigs, pinActivation=pinActivation, 
+            nextProtos=nextProtos, anon=anon)
         for result in self._handshakeWrapperAsync(handshaker, checker):
             yield result
 
 
     def _handshakeServerAsyncHelper(self, verifierDB,
                              certChain, privateKey, reqCert, sessionCache,
-                             settings, reqCAs, tack, breakSigs):
+                             settings, reqCAs, 
+                             tack, breakSigs, pinActivation, 
+                             nextProtos, anon):
 
         self._handshakeStart(client=False)
 
-        if (not verifierDB) and (not certChain):
+        if (not verifierDB) and (not certChain) and not anon:
             raise ValueError("Caller passed no authentication credentials")
         if certChain and not privateKey:
             raise ValueError("Caller passed a certChain but no privateKey")
@@ -896,8 +1043,11 @@ class TLSConnection(TLSRecordLayer):
             raise ValueError("Caller passed reqCAs but not reqCert")            
         if certChain and not isinstance(certChain, X509CertChain):
             raise ValueError("Unrecognized certificate type")
-        if (tack or breakSigs) and not tackpyLoaded:
-            raise ValueError("TACKpy is not loaded")
+        if (tack or breakSigs or pinActivation):
+            if not tackpyLoaded:
+                raise ValueError("tackpy is not loaded")
+            if not settings or not settings.useExperimentalTackExtension:
+                raise ValueError("useExperimentalTackExtension not enabled")
 
         if not settings:
             settings = HandshakeSettings()
@@ -908,7 +1058,8 @@ class TLSConnection(TLSRecordLayer):
         
         # Handle ClientHello and resumption
         for result in self._serverGetClientHello(settings, certChain,\
-                                            verifierDB, sessionCache):
+                                            verifierDB, sessionCache,
+                                            anon):
             if result in (0,1): yield result
             elif result == None:
                 self._handshakeDone(resumed=True)                
@@ -924,19 +1075,22 @@ class TLSConnection(TLSRecordLayer):
         else:
             sessionID = createByteArraySequence([])
         
+        if not clientHello.supports_npn:
+            nextProtos = None
+
         # If not doing a certificate-based suite, discard the TACK
         if not cipherSuite in CipherSuite.certAllSuites:
             tack = None
 
         # Prepare a TACK Extension if requested
         if clientHello.tack:
-            tackExt = TACK_Extension()
-            tackExt.create(tack, breakSigs)
+            tackExt = TackExtension.create(tack, breakSigs, pinActivation)
         else:
             tackExt = None
         serverHello = ServerHello()
         serverHello.create(self.version, getRandomBytes(32), sessionID, \
-                            cipherSuite, CertificateType.x509, tackExt)
+                            cipherSuite, CertificateType.x509, tackExt,
+                            nextProtos)
 
         # Perform the SRP key exchange
         clientCertChain = None
@@ -948,7 +1102,6 @@ class TLSConnection(TLSRecordLayer):
                 else: break
             premasterSecret = result
 
-
         # Perform the RSA key exchange
         elif cipherSuite in CipherSuite.certSuites:
             for result in self._serverCertKeyExchange(clientHello, serverHello, 
@@ -959,10 +1112,22 @@ class TLSConnection(TLSRecordLayer):
                 else: break
             (premasterSecret, clientCertChain) = result
 
-        # Exchange Finished messages
+        # Perform anonymous Diffie Hellman key exchange
+        elif cipherSuite in CipherSuite.anonSuites:
+            for result in self._serverAnonKeyExchange(clientHello, serverHello, 
+                                        cipherSuite, settings):
+                if result in (0,1): yield result
+                else: break
+            premasterSecret = result
+        
+        else:
+            assert(False)
+                        
+        # Exchange Finished messages      
         for result in self._serverFinished(premasterSecret, 
                                 clientHello.random, serverHello.random,
-                                cipherSuite, settings.cipherImplementations):
+                                cipherSuite, settings.cipherImplementations,
+                                nextProtos):
                 if result in (0,1): yield result
                 else: break
         masterSecret = result
@@ -975,7 +1140,7 @@ class TLSConnection(TLSRecordLayer):
             serverCertChain = None
         self.session.create(masterSecret, serverHello.session_id, cipherSuite,
             clientHello.srp_username, clientCertChain, serverCertChain,
-            tackExt)
+            tackExt, serverHello.tackExt!=None, clientHello.server_name)
             
         #Add the session object to the session cache
         if sessionCache and sessionID:
@@ -985,7 +1150,7 @@ class TLSConnection(TLSRecordLayer):
 
 
     def _serverGetClientHello(self, settings, certChain, verifierDB,
-                                sessionCache):
+                                sessionCache, anon):
         #Initialize acceptable cipher suites
         cipherSuites = []
         if verifierDB:
@@ -995,6 +1160,8 @@ class TLSConnection(TLSRecordLayer):
             cipherSuites += CipherSuite.getSrpSuites(settings.cipherNames)
         elif certChain:
             cipherSuites += CipherSuite.getCertSuites(settings.cipherNames)
+        elif anon:
+            cipherSuites += CipherSuite.getAnonSuites(settings.cipherNames)
         else:
             assert(False)
 
@@ -1025,7 +1192,7 @@ class TLSConnection(TLSRecordLayer):
         else:
             #Set the version to the client's version
             self.version = clientHello.client_version  
-        
+
         #If resumption was requested and we have a session cache...
         if clientHello.session_id and sessionCache:
             session = None
@@ -1051,6 +1218,11 @@ class TLSConnection(TLSRecordLayer):
                             for result in self._sendError(\
                                     AlertDescription.handshake_failure):
                                 yield result
+                    if clientHello.server_name:
+                        if clientHello.server_name != session.serverName:
+                            for result in self._sendError(\
+                                    AlertDescription.handshake_failure):
+                                yield result                    
                 except KeyError:
                     pass
 
@@ -1060,7 +1232,7 @@ class TLSConnection(TLSRecordLayer):
                 serverHello = ServerHello()
                 serverHello.create(self.version, getRandomBytes(32),
                                    session.sessionID, session.cipherSuite,
-                                   CertificateType.x509, None)
+                                   CertificateType.x509, None, None)
                 for result in self._sendMsg(serverHello):
                     yield result
 
@@ -1101,7 +1273,6 @@ class TLSConnection(TLSRecordLayer):
                     AlertDescription.handshake_failure,
                     "No mutual ciphersuite"):
                 yield result
-
         if cipherSuite in CipherSuite.srpAllSuites and \
                             not clientHello.srp_username:
             for result in self._sendError(\
@@ -1116,7 +1287,7 @@ class TLSConnection(TLSRecordLayer):
                     AlertDescription.handshake_failure,
                     "the client doesn't support my certificate type"):
                 yield result
-        
+
         # If resumption was not requested, or
         # we have no session cache, or
         # the client's session_id was not found in cache:
@@ -1263,6 +1434,8 @@ class TLSConnection(TLSRecordLayer):
         premasterSecret = privateKey.decrypt(\
             clientKeyExchange.encryptedPreMasterSecret)
 
+        # On decryption failure randomize premaster secret to avoid
+        # Bleichenbacher's "million message" attack
         randomPreMasterSecret = getRandomBytes(48)
         versionCheck = (premasterSecret[0], premasterSecret[1])
         if not premasterSecret:
@@ -1308,20 +1481,67 @@ class TLSConnection(TLSRecordLayer):
         yield (premasterSecret, clientCertChain)
 
 
+    def _serverAnonKeyExchange(self, clientHello, serverHello, cipherSuite, 
+                               settings):
+        # Calculate DH p, g, Xs, Ys
+        dh_p = getRandomSafePrime(32, False)
+        dh_g = getRandomNumber(2, dh_p)        
+        dh_Xs = bytesToNumber(getRandomBytes(32))        
+        dh_Ys = powMod(dh_g, dh_Xs, dh_p)
+
+        #Create ServerKeyExchange
+        serverKeyExchange = ServerKeyExchange(cipherSuite)
+        serverKeyExchange.createDH(dh_p, dh_g, dh_Ys)
+        
+        #Send ServerHello[, Certificate], ServerKeyExchange,
+        #ServerHelloDone  
+        msgs = []
+        msgs.append(serverHello)
+        msgs.append(serverKeyExchange)
+        msgs.append(ServerHelloDone())
+        for result in self._sendMsgs(msgs):
+            yield result
+        
+        #From here on, the client's messages must have the right version
+        self._versionCheck = True
+        
+        #Get and check ClientKeyExchange
+        for result in self._getMsg(ContentType.handshake,
+                                   HandshakeType.client_key_exchange,
+                                   cipherSuite):
+            if result in (0,1):
+                yield result 
+            else:
+                break
+        clientKeyExchange = result
+        dh_Yc = clientKeyExchange.dh_Yc
+        
+        if dh_Yc % dh_p == 0:
+            for result in self._sendError(AlertDescription.illegal_parameter,
+                    "Suspicious dh_Yc value"):
+                yield result
+            assert(False) # Just to ensure we don't fall through somehow            
+
+        #Calculate premaster secre
+        S = powMod(dh_Yc,dh_Xs,dh_p)
+        premasterSecret = numberToBytes(S)
+        
+        yield premasterSecret
+
 
     def _serverFinished(self,  premasterSecret, clientRandom, serverRandom,
-                        cipherSuite, cipherImplementations):
-                        
+                        cipherSuite, cipherImplementations, nextProtos):
         masterSecret = calcMasterSecret(self.version, premasterSecret,
                                       clientRandom, serverRandom)
-                        
+        
         #Calculate pending connection states
         self._calcPendingStates(cipherSuite, masterSecret, 
                                 clientRandom, serverRandom,
                                 cipherImplementations)
 
         #Exchange ChangeCipherSpec and Finished messages
-        for result in self._getFinished(masterSecret):
+        for result in self._getFinished(masterSecret, 
+                        expect_next_protocol=nextProtos is not None):
             yield result
 
         for result in self._sendFinished(masterSecret):
@@ -1353,7 +1573,7 @@ class TLSConnection(TLSRecordLayer):
         for result in self._sendMsg(finished):
             yield result
 
-    def _getFinished(self, masterSecret):
+    def _getFinished(self, masterSecret, expect_next_protocol=False):
         #Get and check ChangeCipherSpec
         for result in self._getMsg(ContentType.change_cipher_spec):
             if result in (0,1):
@@ -1368,6 +1588,19 @@ class TLSConnection(TLSRecordLayer):
         #Switch to pending read state
         self._changeReadState()
 
+        if expect_next_protocol:
+            for result in self._getMsg(ContentType.handshake, HandshakeType.next_protocol):
+                if result in (0,1):
+                    yield result
+            if result is None:
+                for result in self._sendError(AlertDescription.unexpected_message,
+                                             "Didn't get NextProtocol message"):
+                    yield result
+
+            self.next_proto = result.next_proto
+        else:
+            self.next_proto = None
+
         #Calculate verification data
         verifyData = self._calcFinished(masterSecret, False)
 
